#include "Components.h"

#include <iomanip>


Node::Node(int p_ID, const std::array<double, 3>& p_coordinates, bool p_VertexDot) :
    m_ID(p_ID), m_coordinates(p_coordinates), m_vertexDot(p_VertexDot) {}


//  
std::ostream& operator<<(std::ostream& out, const Node& p_node) {
    out << "Node ID: " << p_node.m_ID << "\t";
    out << "Node coordinate: (" << p_node.m_coordinates[0] << ";" << p_node.m_coordinates[1] << ";" << p_node.m_coordinates[2] << ")\t";
    if (p_node.m_vertexDot)
        out << "Node is a vertex\n";
    else
        out << "Node is internal\n";
    out << std::endl;
    return out;
}


std::ostream& operator<<(std::ostream& out, const std::vector<Node>& p_node) {
    for (const auto& node : p_node)
        out << node;
    return out;
};


//  
std::ostream& operator<<(std::ostream& out, const FiniteElement& p_finiteElement) {
    out << "Finite Element ID: " << p_finiteElement.m_FiniteElementID << "\t";
    out << "Geometry Area ID: " << p_finiteElement.m_GeometryAreaID << "\t";
    out << "Nodes of Finite Element: ";
    for (auto& node : p_finiteElement.m_NodeIDs) {
        out << node << " ";
    }
    out << "\n" << std::endl;
    return out;
}


std::ostream& operator<<(std::ostream& out, const std::vector<FiniteElement>& p_finiteElement) {
    for (const auto& node : p_finiteElement)
        out << node;
    return out;
}


Edge::Edge(int firstNodeId, int secNodeId)
    : m_edgeNodes({ firstNodeId, secNodeId }) {}


Edge& Edge::operator=(const Edge& p_edge) {
    if (this != &p_edge) {
        this->m_edgeNodes = p_edge.m_edgeNodes;
        this->m_midNode = p_edge.m_midNode;
    }
    return *this;
}


bool Edge::operator==(const Edge& p_edge) const {
    return ((m_edgeNodes.first == p_edge.m_edgeNodes.first) &&
        (m_edgeNodes.second == p_edge.m_edgeNodes.second)) || ((m_edgeNodes.first == p_edge.m_edgeNodes.second) &&
        (m_edgeNodes.second == p_edge.m_edgeNodes.first));
}



