#pragma once
#include <iostream>
#include <stdlib.h>
#include <vector>
#include <set>
#include <array>
#include "Components.h"
class MeshLoader {
private:
	Node getMiddleNode(const Edge&);
	std::array<double, 3> getMiddleCoords(const Edge&);
protected:
	std::vector<Node> Nodes;
	std::vector<FiniteElement> FiniteElements;
	std::vector<FiniteElement> BoundaryElements;
public:
	virtual void loadMesh(const std::string&) = 0;
	virtual ~MeshLoader() = default;

	const std::vector<Node>& getNodes();
	const std::vector<FiniteElement>& getFiniteElements();
	const std::vector<FiniteElement>& getBoundaryElements();

	std::vector<int> getFiniteElementByID(int, int, int);
	std::vector<int> getFiniteElementByEdge(int, int);
	std::vector<int> getNodesIDByBorder(int);

	std::vector<int> getFiniteElementIdBySurface(int);
	std::vector<int> getBoundElementIDByBorder(int);

	static void printNodes(const std::vector<Node>&);
	static void printFiniteElements(const std::vector<FiniteElement>&);

	void insertNodeMid();

	std::vector<std::set<int>> createNeighborsVector();
};

